@@ -1,4 +1,4 @@
-/* $Id: kildclient.h,v 1.149 2007/05/13 22:00:56 ekalin Exp $ */
+/* $Id: kildclient.h,v 1.160 2007/11/25 18:37:07 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -35,6 +35,10 @@
 #  include <gnutls/gnutls.h>
 #endif
 
+#ifdef __MINGW32__
+#  include <winsock2.h>
+#endif
+
 
 /**********************
  * Limits and defines *
@@ -43,6 +47,7 @@
 #define MAX_TIME_STR 150
 #define VTE_ENC      "UTF-8"
 #define DEFAULT_TERMINAL_FONT "Monospace 10"
+#define DEFAULT_LOGTIME_FORMAT "%c> "
 
 
 /**************
@@ -244,6 +249,7 @@ struct WorldGUI_s
   GtkTextView       *txtView;
   GtkTextBuffer     *txtBuffer;
 
+  GtkBox            *hboxTab;
   GtkLabel          *lblNotebook;
 
   GtkWidget         *search_box;
@@ -299,12 +305,12 @@ struct world_s
   gint             charset_index;
   gchar           *host;
   gchar           *host_ip;
-  guint16          port;
+  gchar           *port;
 
   /* Auto-login data */
-  ConnStyle     connection_style;
-  GtkTreeModel *logon_characters;
-  gint          character_used;
+  ConnStyle        connection_style;
+  GtkTreeModel    *logon_characters;
+  gint             character_used;
 
   /* Display settings */
   char            *terminalfont;
@@ -347,6 +353,7 @@ struct world_s
   /* Input settings */
   gboolean         repeat_commands;
   gboolean         cmd_echo;
+  gboolean         store_commands;
   gint             commands_to_save;
   gint             input_n_lines;
   gint             input_n_lines_saved;
@@ -379,20 +386,29 @@ struct world_s
 
   /* Logging */
   gchar           *log_file_name;
+  gchar           *log_actual_file;
   FILE            *log_file;
   gchar           *log_timeformat;
+  gboolean         log_add_time;
+  gboolean         log_autostart;
 
   /* Basic connection information */
   gboolean         connected;
+  gboolean         connecting;
   gboolean         connected_at_least_once;
-  int              err_no;        /* To display in disconnected dialog */
+  gchar           *errmsg;        /* Error message to display */
   GIOChannel      *iochan;
+#ifndef __MINGW32__
   int              sock;
+#else
+  SOCKET           sock;
+#endif
   guint            io_watch_id;
   guint            timeout_id;
   glong            deleted_lines;
   glong            rawbytes;      /* Bytes received from network */
   glong            bytes;         /* Bytes seen after decompression */
+  struct addrinfo *addrinfo;      /* addrinfo for connecting */
 
   /* Buffers and state */
   TelnetState      telnet_state;             /* State of Telnet IAC FSM */
@@ -723,7 +739,6 @@ extern gnutls_certificate_credentials tls_certif_cred;
  *********************************/
 GtkWidget* create_main_window(WorldGUI *gui);
 World*     get_world_to_connect();
-gboolean   connect_to(World *world, gchar **errmsg);
 void       show_preferences_dialog_cb(GtkWidget *widget, gpointer data);
 void       remove_world(World *world, gboolean remove_gui);
 gboolean   remove_world_timer(World *world);
@@ -834,6 +849,9 @@ void       fill_we_statusbar_tab(World *world, GladeXML *gladexml);
 GtkWidget *get_we_input_tab(GladeXML *gladexml);
 void       fill_we_input_tab(World *world, GladeXML *gladexml);
 void       update_we_input_parameters(World *world);
+GtkWidget *get_we_logging_tab(GladeXML *gladexml, World *world);
+void       fill_we_logging_tab(World *world, GladeXML *gladexml);
+void       update_we_logging_parameters(World *world);
 void       set_input_line_controls(World *world,
                                    GtkWidget *radSingleLine,
                                    GtkWidget *radMultiLine,
@@ -880,6 +898,10 @@ void       we_guied_bool_col_toggled_cb(GtkCellRendererToggle *renderer,
 void       we_guied_selection_changed_cb(GtkTreeSelection *selection,
                                          gpointer          data);
 
+GtkWidget *we_guied_confirm_delete_dialog_new(GtkWindow   *parent,
+                                              int          n,
+                                              const gchar *singular,
+                                              const gchar *plural);
 void       we_lplugin_insert_plugin(World *world, Plugin *plugin);
 void       we_lplugin_update_plugin(World *world, Plugin *plugin);
 
@@ -931,13 +953,11 @@ void do_multi_line_send(MLSendData *mlcontext, gdouble delay);
 /*********************************
  * Function prototypes - Network *
  *********************************/
+gboolean  connect_to(World *world, gchar **errmsg);
 gboolean  disconnect_world(gpointer data);
 gboolean  reconnect_world(gpointer data);
 gboolean  connect_another(gpointer data);
 gchar    *close_connection(World *world);
-gboolean  data_ready_cb(GIOChannel   *iochan,
-                        GIOCondition  cond,
-                        gpointer      data);
 gboolean  flush_buffer_timeout_cb(gpointer data);
 void      perform_auto_logon(World *world);
 
@@ -1062,8 +1082,9 @@ GtkTreeModel *get_hook_model(World *world, const gchar *hookname);
 /*********************************
  * Function Prototypes - Logging *
  *********************************/
-void  write_log_line(World *world, const gchar *line);
-char *close_log_file(World *world, int has_timeformat);
+void     write_log_line(World *world, const gchar *line);
+gboolean start_log(World *world, gchar **errormsg);
+void     stop_log(World *world);
 
 
 /*********************************
