@@ -1,4 +1,4 @@
-/* $Id: log.c,v 1.11 2005/11/03 19:42:42 ekalin Exp $ */
+/* $Id: log.c,v 1.12 2007/11/24 19:14:39 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #endif
 
 #include <string.h>
+#include <unistd.h>
 #include <time.h>
 #include <sys/time.h>
 #include <libintl.h>
@@ -38,11 +39,6 @@
  *************************/
 
 
-/***************************
- * Telnet protocol defines *
- ***************************/
-
-
 /***********************
  * Function prototypes *
  ***********************/
@@ -55,7 +51,7 @@ write_log_line(World *world, const char *line)
   int         len;
   static char timestr[MAX_TIME_STR];
 
-  if (world->log_timeformat) {
+  if (world->log_add_time) {
     time_t     currtime;
     struct tm *brokentime;
 
@@ -82,31 +78,59 @@ write_log_line(World *world, const char *line)
 }
 
 
-char *
-close_log_file(World *world, int keep_timeformat)
+gboolean
+start_log(World *world, gchar **errmsg)
 {
-  /* Returns the formatted time string, but this is only valid until
-     the next call. */
+  time_t       curr_time;
+  struct tm   *brokentime;
+  static char  actualfile[PATH_MAX+1];
+  static char  timestr[MAX_TIME_STR];
 
+  if (!world->log_file_name) {
+    *errmsg = g_strdup(_("File name not specified."));
+    return FALSE;
+  }
+
+  /* Create file with possible date and time sequences. */
+  time(&curr_time);
+  brokentime = localtime(&curr_time);
+  strftime(actualfile, PATH_MAX, world->log_file_name, brokentime);
+
+  if ((world->log_file = fopen(actualfile, "a")) != NULL) {
+    world->log_actual_file = g_strdup(actualfile);
+
+    /* Use unbuffered output */
+    setbuf(world->log_file, NULL);
+
+    strcpy(timestr, ctime(&curr_time));
+    /* Remove final nl */
+    timestr[strlen(timestr) - 1] = '\0';
+    fprintf(world->log_file, "*** LOG STARTED AT %s ***\n", timestr);
+
+    return TRUE;
+  } else {
+    *errmsg = g_strdup(_("Could not open log file."));
+    return FALSE;
+  }
+}
+
+
+void
+stop_log(World *world)
+{
   time_t      curr_time;
   static char timestr[MAX_TIME_STR];
 
-  time(&curr_time);
-  strcpy(timestr, ctime(&curr_time));
-  /* Remove final nl */
-  timestr[strlen(timestr) - 1] = '\0';
   if (world->log_file) {
+    time(&curr_time);
+    strcpy(timestr, ctime(&curr_time));
+    /* Remove final nl */
+    timestr[strlen(timestr) - 1] = '\0';
     fprintf(world->log_file, "\n*** LOG ENDED AT %s ***\n\n",
             timestr);
     fclose(world->log_file);
     world->log_file = NULL;
-    g_free(world->log_file_name);
-
-    if (!keep_timeformat) {
-      g_free(world->log_timeformat);
-      world->log_timeformat = NULL;
-    }
+    g_free(world->log_actual_file);
+    world->log_actual_file = NULL;
   }
-
-  return timestr;
 }
