@@ -1,4 +1,4 @@
-/* $Id: main.c,v 1.44 2006/07/15 20:32:57 ekalin Exp $ */
+/* $Id: main.c,v 1.49 2007/11/25 18:19:36 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -125,7 +125,7 @@ main(int argc, char *argv[], char *env[])
   g_option_context_free(opt_context);
   if (wants_version) {
     printf(PACKAGE_NAME " " PACKAGE_VERSION "\n");
-    printf(_("Written by Eduardo M Kalinowski (ekalin@bol.com.br)\n"));
+    printf(_("Written by Eduardo M Kalinowski (ekalin@gmail.com)\n"));
     exit(0);
   }
   if (dumpfile) {
@@ -153,6 +153,7 @@ main(int argc, char *argv[], char *env[])
   }
 #endif
 
+  g_thread_init(NULL);
   gtk_init(&argc, &argv);
   gtk_rc_parse_string("gtk-entry-select-on-focus=0");
 
@@ -217,7 +218,8 @@ main(int argc, char *argv[], char *env[])
   while (open_worlds) {
     World *world = (World *) open_worlds->data;
 
-    if (world->connected) {
+    if (world->connected || world->connecting) {
+      /* This leaks the return value, but we are exiting the program. */
       close_connection(world);
     }
     save_world_to_file(world);
@@ -262,7 +264,8 @@ confirm_quit(void)
   worldptr = open_worlds;
   while (worldptr) {
     World *world = (World *) worldptr->data;
-    if (world->connected && !world->hooks.OnCloseConnected) {
+    if ((world->connected || world->connecting)
+	&& !world->hooks.OnCloseConnected) {
       ++n_open;
     }
     worldptr = worldptr->next;
@@ -286,7 +289,7 @@ confirm_quit(void)
                                     _("There are %d open worlds. Do you really want to quit?"),
                                     n_open);
   }
-
+  gtk_window_set_title(GTK_WINDOW(msgdlg), _("Really Quit KildClient?"));
   gtk_dialog_add_button(GTK_DIALOG(msgdlg),
                         _("Don't quit"), GTK_RESPONSE_NO);
   dlgbutton = gtk_button_new_from_stock(GTK_STOCK_QUIT);
