@@ -1,4 +1,4 @@
-/* $Id: mainwindow.c,v 1.122 2006/07/30 21:51:38 ekalin Exp $ */
+/* $Id: mainwindow.c,v 1.130 2007/11/18 20:10:39 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -63,15 +63,12 @@ gboolean   debug_matches        = 0;
  * File global variables *
  *************************/
 static GtkWidget *ntbkWorlds;
-#ifdef HAVE_LIBGNUTLS
-const int cert_type_priority[3] = { GNUTLS_CRT_X509, GNUTLS_CRT_OPENPGP, 0 };
-#endif
 
 
 /***********************
  * Function prototypes *
  ***********************/
-static void     update_world_with_focus(gint n);
+static gboolean update_world_with_focus(gpointer data);
 static gint     attach_world_gui(WorldGUI *gui);
 static void     detach_world_gui(WorldGUI *gui);
 static gchar   *format_time_string(World *world, int ctime, int itime);
@@ -90,6 +87,10 @@ G_MODULE_EXPORT void     notebook_page_changed_cb(GtkNotebook     *notebook,
                                                   GtkNotebookPage *page,
                                                   guint            page_num,
                                                   gpointer         data);
+G_MODULE_EXPORT void     notebook_page_reordered_cb(GtkNotebook *notebook,
+                                                    GtkWidget   *page,
+                                                    guint        page_num,
+                                                    gpointer     data);
 /* Menus */
 G_MODULE_EXPORT void     menu_world_open_cb(GtkWidget *widget,
                                             gpointer data);
@@ -218,204 +219,6 @@ create_main_window(WorldGUI *gui)
 }
 
 
-gboolean
-connect_to(World *world, gchar **errmsg)
-{
-  int                sock;
-  struct sockaddr_in servername;
-  guint32            addr;
-  struct hostent    *hostinfo;
-  gchar             *status_text;
-  gchar             *msg;
-
-#ifndef HAVE_LIBGNUTLS
-  /* Oops, we cannot do that */
-  if (world->use_tls) {
-    GtkWidget *msgdlg;
-    GtkWidget *dlgbutton;
-    GtkWidget *dlgbuttonimage;
-
-    msgdlg = gtk_message_dialog_new(GTK_WINDOW(world->dlgEditWorld),
-                                    GTK_DIALOG_MODAL,
-                                    GTK_MESSAGE_QUESTION,
-                                    GTK_BUTTONS_NONE,
-                                    _("You're attempting to connect to a world that uses SSL, but this version of KildClient has been built without SSL support. Try to connect without SSL?"));
-    gtk_dialog_add_button(GTK_DIALOG(msgdlg),
-                          _("Connect without SSL"), GTK_RESPONSE_YES);
-    dlgbutton = gtk_button_new_with_label(_("Do not connect"));
-    dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_CANCEL,
-                                              GTK_ICON_SIZE_MENU);
-    gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-    gtk_widget_show_all(dlgbutton);
-    gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                                 dlgbutton, GTK_RESPONSE_NO);
-    GTK_WIDGET_SET_FLAGS(dlgbutton, GTK_CAN_DEFAULT);
-    gtk_dialog_set_default_response(GTK_DIALOG(msgdlg), GTK_RESPONSE_NO);
-    //gtk_widget_grab_default(dlgbutton);
-  if (gtk_dialog_run(GTK_DIALOG(msgdlg)) == GTK_RESPONSE_YES) {
-      world->use_tls = FALSE;
-      gtk_widget_destroy(msgdlg);
-    } else {
-      *errmsg = g_strdup(_("This version of KildClient does not have SSL support."));
-      gtk_widget_destroy(msgdlg);
-      return FALSE;
-    }
-  }
-#else /* defined HAVE_LIBGNUTLS */
-  if (world->use_tls) {
-    gnutls_init(&world->tls_session, GNUTLS_CLIENT);
-    gnutls_set_default_priority(world->tls_session);
-    gnutls_certificate_type_set_priority(world->tls_session,
-                                         cert_type_priority);
-    gnutls_credentials_set(world->tls_session,
-                           GNUTLS_CRD_CERTIFICATE, tls_certif_cred);
-  }
-#endif
-
-  /* Create host/port structure */
-  servername.sin_family = AF_INET;
-  servername.sin_port = htons(world->port);
-  hostinfo = gethostbyname(world->host);
-  if (hostinfo == NULL) {
-    *errmsg = g_strdup_printf(_("Unknown host %s."),
-                              world->host);
-    return FALSE;
-  }
-  servername.sin_addr = *(struct in_addr *) hostinfo->h_addr;
-  addr = servername.sin_addr.s_addr;
-
-  if (!world->connected_at_least_once) {
-    configure_gui(world->gui, world);
-  }
-
-  /* Create IP address representation */
-  if (!world->connected_at_least_once) {
-    world->host_ip = g_strdup_printf("%d.%d.%d.%d",
-                                     (addr      ) & 0xFF,
-                                     (addr >>  8) & 0xFF,
-                                     (addr >> 16) & 0xFF,
-                                     (addr >> 24) & 0xFF);
-  }
-
-  msg = g_strdup_printf(_("Attempting to connect%s to host %s (%s) port %d.\n"),
-                        world->use_tls ? _(" with SSL") : "",
-                        world->host, world->host_ip, world->port);
-  ansitextview_append_string_with_fgcolor(world->gui, msg,
-                                          globalPrefs.idxInfoMsgColor);
-  g_free(msg);
-
-  while (gtk_events_pending())
-    gtk_main_iteration();
-
-  sock = socket(PF_INET, SOCK_STREAM, 0);
-  if (sock < 0) {
-    *errmsg = g_strdup_printf(_("Could not create socket: %s"),
-                              g_strerror(errno));
-    return FALSE;
-  }
-
-  if (connect(sock,
-              (struct sockaddr *) &servername,
-              sizeof(servername)) < 0) {
-    *errmsg = g_strdup_printf(_("Could not connect to host %s port %d: %s"),
-                              world->host, world->port,
-                              g_strerror(errno));
-    return FALSE;
-  }
-
-#ifdef HAVE_LIBGNUTLS
-  if (world->use_tls) {
-    int ret;
-
-    gnutls_transport_set_ptr(world->tls_session,
-#ifndef __x86_64      /* This is to remove a warning about pointer sizes */
-                             (gnutls_transport_ptr) sock);
-#else
-                             (gnutls_transport_ptr) (long) sock);
-#endif
-    /* Perform the TLS handshake */
-    ret = gnutls_handshake(world->tls_session);
-    if (ret < 0) {
-      *errmsg = g_strdup_printf(_("SSL Handshake failed: %s"),
-                                gnutls_strerror(ret));
-      gnutls_deinit(world->tls_session);
-      return FALSE;
-    }
-  }
-#endif
-
-  status_text = g_strdup_printf(_("Connected%s to host %s (%s) port %d."),
-                                world->use_tls ? _(" with SSL") : "",
-                                world->host, world->host_ip, world->port);
-  ansitextview_append_string_with_fgcolor(world->gui, status_text,
-                                          globalPrefs.idxInfoMsgColor);
-  ansitextview_append_string(world->gui, "\n");
-  gtk_label_set_text(world->gui->lblStatus, status_text);
-  g_free(status_text);
-
-  gtk_label_set_text(world->gui->lblNotebook, world->name);
-
-  world->noecho = FALSE;
-  world->sock = sock;
-  world->iochan = g_io_channel_unix_new(sock);
-  /* We'll do character code handling later */
-  g_io_channel_set_encoding(world->iochan, NULL, NULL);
-  world->connected = TRUE;
-
-  world->io_watch_id = g_io_add_watch_full(world->iochan,
-                                           G_PRIORITY_HIGH,
-                                           G_IO_IN | G_IO_HUP | G_IO_ERR,
-                                           data_ready_cb,
-                                           world,
-                                           NULL);
-
-  currentWorld = world;
-  world_for_perl = world;
-  if (!world->connected_at_least_once) {
-    init_perl_script(world);
-    load_startup_plugins(world);
-    if (world->scriptfile) {
-      perl_script_import_file(world->perl_interpreter, world->scriptfile);
-    }
-    load_command_history(world);
-    load_permanent_variables(world);
-  } else {
-    if (world->has_unread_text) {
-      world->has_unread_text = 0;
-      --worlds_with_new_text;
-      adjust_window_title();
-    }
-  }
-
-  if (!world->connected_at_least_once) {
-    trigger_precompute_res(world);
-    alias_precompute_res(world);
-  }
-
-  perform_auto_logon(world);
-  execute_hook(world, "OnConnect", NULL);
-
-  if (!world->connected_at_least_once) {
-    start_timers(world);
-  }
-
-  if (!world->connected_at_least_once) {
-    open_worlds = g_list_append(open_worlds, world);
-  }
-
-  world->connected_at_least_once = TRUE;
-  world->connected_time    = time(NULL);
-  world->last_command_time = time(NULL);
-  world->rawbytes = world->bytes = 0;
-
-  /* Set it as the last opened one */
-  g_free(globalPrefs.last_open_world);
-  globalPrefs.last_open_world = g_strdup(world->name);
-
-  return TRUE;
-}
-
-
 gboolean
 exit_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
@@ -534,15 +337,45 @@ notebook_page_changed_cb(GtkNotebook     *notebook,
     }
   }
 
-  /* Updates currentWorld, handles focus, etc */
-  update_world_with_focus(page_num);
+  /* Updates currentWorld, handles focus, etc.
+     Called as idle because the change may result from a call to a Perl
+     function, and this changes the current Perl interpreter. */
+  g_idle_add(update_world_with_focus, GINT_TO_POINTER(page_num));
+}
+
+
+void
+notebook_page_reordered_cb(GtkNotebook *notebook,
+                           GtkWidget   *widget,
+                           guint        new_pos,
+                           gpointer     data)
+{
+  World    *world;
+  WorldGUI *gui;
+  gint      old_pos;
+  GList    *sucessor;
+
+  gui = (WorldGUI *) g_object_get_data(G_OBJECT(widget), "gui");
+  world = gui->world;
+
+  old_pos = g_list_index(open_worlds, world);
+
+  sucessor = g_list_nth(open_worlds, new_pos);
+  if (old_pos < new_pos) {
+    sucessor = g_list_next(sucessor);
+  }
+
+  open_worlds = g_list_remove(open_worlds, world);
+  open_worlds = g_list_insert_before(open_worlds, sucessor, world);
 }
 
 
 static
-void
-update_world_with_focus(gint n)
+gboolean
+update_world_with_focus(gpointer data)
 {
+  gint n = GPOINTER_TO_INT(data);
+
   /* Now we make it point to the new world */
   currentWorld = (World *) g_list_nth_data(open_worlds, n);
   world_for_perl = currentWorld;
@@ -562,6 +395,9 @@ update_world_with_focus(gint n)
       execute_hook(currentWorld, "OnGetFocus", NULL);
     }
   }
+
+  /* Do not run again */
+  return FALSE;
 }
 
 
@@ -693,21 +529,28 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
 
   /* Basic information */
   gtk_label_set_text(GTK_LABEL(lblName), currentWorld->name);
-  str = g_strdup_printf("%s (%s)", currentWorld->host, currentWorld->host_ip);
+  if (currentWorld->host_ip) {
+    str = g_strdup_printf("%s (%s)", currentWorld->host, currentWorld->host_ip);
+  } else {
+    str = g_strdup(currentWorld->host);
+  }
   gtk_label_set_text(GTK_LABEL(lblHost), str);
   g_free(str);
-  str = g_strdup_printf("%d", currentWorld->port);
-  gtk_label_set_text(GTK_LABEL(lblPort), str);
-  g_free(str);
+  gtk_label_set_text(GTK_LABEL(lblPort), currentWorld->port);
 
-  now = time(NULL);
-  timestr = g_string_new(NULL);
-  format_hms_time(timestr, difftime(now, currentWorld->connected_time));
-  gtk_label_set_text(GTK_LABEL(lblConnTime), timestr->str);
-  g_string_truncate(timestr, 0);
-  format_hms_time(timestr, difftime(now, currentWorld->last_command_time));
-  gtk_label_set_text(GTK_LABEL(lblIdleTime), timestr->str);
-  g_string_free(timestr, TRUE);
+  if (currentWorld->connected) {
+    now = time(NULL);
+    timestr = g_string_new(NULL);
+    format_hms_time(timestr, difftime(now, currentWorld->connected_time));
+    gtk_label_set_text(GTK_LABEL(lblConnTime), timestr->str);
+    g_string_truncate(timestr, 0);
+    format_hms_time(timestr, difftime(now, currentWorld->last_command_time));
+    gtk_label_set_text(GTK_LABEL(lblIdleTime), timestr->str);
+    g_string_free(timestr, TRUE);
+  } else {
+    gtk_label_set_text(GTK_LABEL(lblConnTime), _("Not connected"));
+    gtk_label_set_text(GTK_LABEL(lblIdleTime), _("Not connected"));
+  }
 
   /* Compression Information */
   if (currentWorld->zstream) {
@@ -928,7 +771,11 @@ attach_world_gui(WorldGUI *gui)
   gint page;
 
   page = gtk_notebook_append_page(GTK_NOTEBOOK(ntbkWorlds), gui->vbox,
-                                  GTK_WIDGET(gui->lblNotebook));
+                                  GTK_WIDGET(gui->hboxTab));
+  gtk_widget_show_all(GTK_WIDGET(gui->hboxTab));
+  gtk_notebook_set_tab_reorderable(GTK_NOTEBOOK(ntbkWorlds),
+                                   gui->vbox,
+                                   TRUE);
 
   gtk_widget_show_all(ntbkWorlds);
 
@@ -947,7 +794,7 @@ detach_world_gui(WorldGUI *gui)
   gtk_notebook_remove_page(GTK_NOTEBOOK(ntbkWorlds), old_page);
 
   new_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(ntbkWorlds));
-  update_world_with_focus(new_page);
+  update_world_with_focus(GINT_TO_POINTER(new_page));
 }
 
 
@@ -970,7 +817,7 @@ remove_world(World *world, gboolean remove_gui)
 {
   open_worlds = g_list_remove(open_worlds, world);
 
-  if (world->connected) {
+  if (world->connected || world->connecting) {
     g_free(close_connection(world));
   }
 
@@ -1021,7 +868,7 @@ menu_world_open_cb(GtkWidget *widget,
   mnuDisconnect = glade_xml_get_widget(gladexml,
                                        "mnuDisconnect");
 
-  if (currentWorld && currentWorld->connected) {
+  if (currentWorld && (currentWorld->connected || currentWorld->connecting)) {
     gtk_widget_set_sensitive(mnuDisconnect, TRUE);
   } else {
     gtk_widget_set_sensitive(mnuDisconnect, FALSE);
