@@ -1,4 +1,4 @@
-/* $Id: net.c,v 1.92 2007/05/13 22:00:56 ekalin Exp $ */
+/* $Id: net.c,v 1.103 2007/11/25 18:19:36 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -32,9 +32,18 @@
 #include <glade/glade.h>
 #include <zlib.h>
 
-#ifdef __MINGW32__
-#  include <windows.h>
+#ifndef __MINGW32__
+#  include <fcntl.h>
+#  include <sys/types.h>
+#  include <sys/socket.h>
+#  include <netdb.h>
+#  include <arpa/inet.h>
+#else /* defined __MINGW32__ */
+/* This is so we can use getaddrinfo() et al. The code will only run
+   in Windows XP. */
+#  define _WIN32_WINNT 0x0501
 #  include <winsock2.h>
+#  include <ws2tcpip.h>
 #endif
 
 #include "kildclient.h"
@@ -47,8 +56,23 @@
  *************************/
 #define MAX_US 250000
 static char *terminal_type = "KildClient";
+#ifdef HAVE_LIBGNUTLS
+const int cert_type_priority[3] = { GNUTLS_CRT_X509, GNUTLS_CRT_OPENPGP, 0 };
+#endif
 
 
+/**************
+ * Data types *
+ **************/
+struct resolve_data_s
+{
+  World           *world;
+  gchar           *host;
+  gchar           *port;
+  struct addrinfo *addrinfo;
+  int              ret;
+};
+
 
 /***************************
  * Telnet protocol defines *
@@ -76,7 +100,16 @@ static char *terminal_type = "KildClient";
 /***********************
  * Function prototypes *
  ***********************/
+static gpointer resolve_name_thread(gpointer data);
+static gboolean continue_connecting_cb(gpointer data);
+static gboolean connect_ready_cb(GIOChannel   *iochan,
+                                 GIOCondition  cond,
+                                 gpointer      data);
+static gboolean connected_to(World *world);
 static void     disconnected_decide_action(World *world, gint action);
+static gboolean data_ready_cb(GIOChannel   *iochan,
+                              GIOCondition  cond,
+                              gpointer      data);
 static gboolean flush_buffer(World *world, int action);
 static void     process_recv_line(World *world,
                                   int start, int end,
@@ -94,6 +127,449 @@ static ssize_t kc_send(World *world, const void *buf, size_t s);
 #define kc_send(w, b, s) (send((w)->sock, (b), (s), 0))
 #endif
 
+#ifndef __MINGW32__
+#  define socket_valid(a) ((a) != -1)
+#else
+#  define socket_valid(a) ((a) != INVALID_SOCKET)
+#endif
+
+
+gboolean
+connect_to(World *world, gchar **errmsg)
+{
+  gchar *msg;
+  struct resolve_data_s *resolve_data;
+
+#ifndef HAVE_LIBGNUTLS
+  /* Oops, we cannot do that */
+  if (world->use_tls) {
+    GtkWidget *msgdlg;
+    GtkWidget *dlgbutton;
+    GtkWidget *dlgbuttonimage;
+
+    msgdlg = gtk_message_dialog_new(GTK_WINDOW(world->dlgEditWorld),
+                                    GTK_DIALOG_MODAL,
+                                    GTK_MESSAGE_QUESTION,
+                                    GTK_BUTTONS_NONE,
+                                    _("You're attempting to connect to a world that uses SSL, but this version of KildClient has been built without SSL support. Try to connect without SSL?"));
+    gtk_window_set_title(GTK_WINDOW(msgdlg), _("KildClient Question"));
+    gtk_dialog_add_button(GTK_DIALOG(msgdlg),
+                          _("Connect without SSL"), GTK_RESPONSE_YES);
+    dlgbutton = gtk_button_new_with_label(_("Do not connect"));
+    dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_CANCEL,
+                                              GTK_ICON_SIZE_MENU);
+    gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
+    gtk_widget_show_all(dlgbutton);
+    gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
+                                 dlgbutton, GTK_RESPONSE_NO);
+    GTK_WIDGET_SET_FLAGS(dlgbutton, GTK_CAN_DEFAULT);
+    gtk_dialog_set_default_response(GTK_DIALOG(msgdlg), GTK_RESPONSE_NO);
+    //gtk_widget_grab_default(dlgbutton);
+  if (gtk_dialog_run(GTK_DIALOG(msgdlg)) == GTK_RESPONSE_YES) {
+      world->use_tls = FALSE;
+      gtk_widget_destroy(msgdlg);
+    } else {
+      *errmsg = g_strdup(_("This version of KildClient does not have SSL support."));
+      gtk_widget_destroy(msgdlg);
+      return FALSE;
+    }
+  }
+#else /* defined HAVE_LIBGNUTLS */
+  if (world->use_tls) {
+    gnutls_init(&world->tls_session, GNUTLS_CLIENT);
+    gnutls_set_default_priority(world->tls_session);
+    gnutls_certificate_type_set_priority(world->tls_session,
+                                         cert_type_priority);
+    gnutls_credentials_set(world->tls_session,
+                           GNUTLS_CRD_CERTIFICATE, tls_certif_cred);
+  }
+#endif
+
+  /* Set up gui */
+  if (!world->connected_at_least_once) {
+    configure_gui(world->gui, world);
+  }
+  gtk_label_set_text(world->gui->lblNotebook, world->name);
+
+  if (g_list_index(open_worlds, world) == -1) {
+    open_worlds = g_list_append(open_worlds, world);
+  }
+
+  world->rawbytes = world->bytes = 0;
+
+  /* Set it as the last opened one */
+  g_free(globalPrefs.last_open_world);
+  globalPrefs.last_open_world = g_strdup(world->name);
+
+  currentWorld = world;
+  world_for_perl = world;
+  if (!world->connected_at_least_once) {
+    init_perl_script(world);
+    load_startup_plugins(world);
+    if (world->scriptfile) {
+      perl_script_import_file(world->perl_interpreter, world->scriptfile);
+    }
+    load_command_history(world);
+    load_permanent_variables(world);
+  } else {
+    if (world->has_unread_text) {
+      world->has_unread_text = 0;
+      --worlds_with_new_text;
+      adjust_window_title();
+    }
+  }
+
+  if (!world->connected_at_least_once) {
+    trigger_precompute_res(world);
+    alias_precompute_res(world);
+  }
+
+  if (!world->connected_at_least_once) {
+    start_timers(world);
+  }
+
+  world->connected_at_least_once = TRUE;
+
+  msg = g_strdup_printf(_("Resolving host %s..."), world->host);
+  gtk_label_set_text(world->gui->lblStatus, msg);
+  ansitextview_append_string_with_fgcolor(world->gui, msg,
+                                          globalPrefs.idxInfoMsgColor);
+  ansitextview_append_string(world->gui, "\n");
+  g_free(msg);
+
+  world->connecting = TRUE;
+
+  /* Spawn a thread to resolve the host name */
+  resolve_data = g_new(struct resolve_data_s, 1);
+  resolve_data->world = world;
+  resolve_data->host  = g_strdup(world->host);
+  resolve_data->port  = g_strdup(world->port);
+  if (!g_thread_create(resolve_name_thread, resolve_data, FALSE, NULL)) {
+    *errmsg = g_strdup(_("Could not start name-resolution thread."));
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
+
+static
+gpointer
+resolve_name_thread(gpointer data)
+{
+  struct resolve_data_s *resolve_data = (struct resolve_data_s *) data;
+  struct addrinfo  hints;
+
+  memset(&hints, 0, sizeof(struct addrinfo));
+  hints.ai_family   = AF_UNSPEC;
+  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_protocol = 0;
+  hints.ai_flags    = 0;
+
+  resolve_data->ret = getaddrinfo(resolve_data->host, resolve_data->port,
+                                  &hints,
+                                  &resolve_data->addrinfo);
+  g_idle_add(continue_connecting_cb, resolve_data);
+
+  return NULL;
+}
+
+
+static
+gboolean
+continue_connecting_cb(gpointer data)
+{
+  struct resolve_data_s *resolve_data = (struct resolve_data_s *) data;
+  World *world = resolve_data->world;
+  gchar *msg;
+  int    ret;
+#ifndef __MINGW32__
+  int    flags;
+#else /* defined __MINGW32__ */
+  u_long flags;
+#endif
+
+  g_free(resolve_data->host);
+  g_free(resolve_data->port);
+
+  if (resolve_data->ret != 0) {
+    /* It could happen that the world has been closed */
+    if (g_list_index(open_worlds, world) != -1) {
+      gchar *msg;
+      gsize  len;
+
+      msg = g_locale_to_utf8(gai_strerror(resolve_data->ret), -1,
+                             NULL, &len, NULL);
+      world->errmsg = g_strdup_printf(_("Could not resolve host %s: %s"),
+                                      world->host, msg);
+      g_free(msg);
+      disconnect_world(world);
+    }
+    g_free(resolve_data);
+
+    /* Only run once */
+    return FALSE;
+  }
+
+  world->addrinfo = resolve_data->addrinfo;
+  g_free(resolve_data);
+
+  /* Create IP address representation */
+  if (!world->host_ip) {
+    world->host_ip = g_malloc(INET6_ADDRSTRLEN);
+    /* Just in case it fails... */
+    strcpy(world->host_ip, "?");
+    getnameinfo(world->addrinfo->ai_addr, world->addrinfo->ai_addrlen,
+                world->host_ip, INET6_ADDRSTRLEN,
+                NULL, 0,
+                NI_NUMERICHOST);
+  }
+
+  msg = g_strdup_printf(_("Attempting to connect%s to host %s (%s) port %s..."),
+                        world->use_tls ? _(" with SSL") : "",
+                        world->host, world->host_ip, world->port);
+  ansitextview_append_string_with_fgcolor(world->gui, msg,
+                                          globalPrefs.idxInfoMsgColor);
+  ansitextview_append_string(world->gui, "\n");
+  gtk_label_set_text(world->gui->lblStatus, msg);
+  g_free(msg);
+
+  while (gtk_events_pending())
+    gtk_main_iteration();
+
+  world->sock = socket(world->addrinfo->ai_family,
+                       world->addrinfo->ai_socktype,
+                       world->addrinfo->ai_protocol);
+  if (!socket_valid(world->sock)) {
+#ifndef __MINGW32__
+    world->errmsg = g_strdup_printf(_("Could not create socket: %s"),
+                                    g_strerror(errno));
+#else /* defined __MINGW32__ */
+    world->errmsg = g_strdup_printf(_("Could not create socket: error #%d"),
+                                    WSAGetLastError());
+#endif
+    disconnect_world(world);
+    return FALSE;
+  }
+#ifndef __MINGW32__
+  flags = fcntl(world->sock, F_GETFL);
+  flags |= O_NONBLOCK;
+  fcntl(world->sock, F_SETFL, flags);
+#else /* defined __MINGW32__ */
+  flags = 1;
+  ioctlsocket(world->sock, FIONBIO, &flags);
+#endif
+
+  ret = connect(world->sock,
+                world->addrinfo->ai_addr, world->addrinfo->ai_addrlen);
+  if (ret < 0) {
+#ifndef __MINGW32__
+    if (errno != EINPROGRESS && errno != EALREADY) {
+      world->errmsg = g_strdup_printf(_("Could not connect to host %s port %s: %s"),
+                                      world->host, world->port,
+                                      g_strerror(errno));
+      return FALSE;
+    }
+#else /* defined __MINGW32__ */
+    int error;
+
+    error = WSAGetLastError();
+    if (error != WSAEWOULDBLOCK) {
+      world->errmsg = g_strdup_printf(_("Could not connect to host %s port %s: Error #%d"),
+                                      world->host, world->port,
+                                      error);
+      return FALSE;
+    }
+#endif
+  }
+
+  world->noecho = FALSE;
+#ifndef __MINGW32__
+  world->iochan = g_io_channel_unix_new(world->sock);
+#else  /* defined __MINGW32__ */
+  world->iochan = g_io_channel_win32_new_socket(world->sock);
+#endif
+  world->io_watch_id = g_io_add_watch(world->iochan,
+                                      G_IO_OUT | G_IO_ERR,
+                                      connect_ready_cb,
+                                      world);
+  /* We'll do character code handling later */
+  g_io_channel_set_encoding(world->iochan, NULL, NULL);
+
+  if (!world->connecting) {
+    connected_to(world);
+  }
+
+  /* This is only called once. */
+  return FALSE;
+}
+
+
+#ifndef __MINGW32__
+static
+gboolean
+connect_ready_cb(GIOChannel   *iochan,
+                 GIOCondition  cond,
+                 gpointer      data)
+{
+  World *world;
+  int    ret;
+
+  world = (World *) data;
+
+  ret = connect(world->sock,
+                world->addrinfo->ai_addr, world->addrinfo->ai_addrlen);
+  if (ret < 0) {
+    if (errno == EALREADY || ret == EINPROGRESS) {
+      return TRUE;  /* Still connecting. */
+    }
+
+    world->errmsg = g_strdup_printf(_("Unable to connect to host %s: %s."),
+                                    world->host, g_strerror(errno));
+
+    disconnect_world(world);
+    return FALSE;
+  } else {
+    connected_to(world);
+    return FALSE;
+  }
+}
+#else /* defined __MINGW32__ */
+gboolean
+connect_ready_cb(GIOChannel   *iochan,
+                 GIOCondition  cond,
+                 gpointer      data)
+{
+  World *world;
+  int    ret;
+  int    error;
+  int    sockerror = 0;
+  int    optlen = sizeof(sockerror);
+
+  world = (World *) data;
+
+  /* Apparently, under Windows it always fails */
+  ret = connect(world->sock,
+                world->addrinfo->ai_addr, world->addrinfo->ai_addrlen);
+  error = WSAGetLastError();
+  if (error == WSAEISCONN) { /* Connection succeeded */
+    connected_to(world);
+    return FALSE;
+  }
+
+  ret = getsockopt(world->sock, SOL_SOCKET, SO_ERROR, &sockerror, &optlen);
+  if (ret != 0) {
+    sockerror = WSAGetLastError();
+  }
+  if (sockerror) {
+    world->errmsg = g_strdup_printf(_("Unable to connect to host %s: Error #%d."),
+                                    world->host, sockerror);
+    disconnect_world(world);
+    return FALSE;
+  }
+
+  if (error == WSAEALREADY || error == WSAEINVAL || error == WSAEWOULDBLOCK) {
+    return TRUE;  /* Still connecting. */
+  }
+
+  /* This should never be reached. But under Windows, it is better
+     to be safe. */
+  g_warning("Unreachable code in connect_read_cb.");
+  world->errmsg = g_strdup_printf(_("Unable to connect to host %s: Error #%d."),
+                                  world->host, sockerror);
+  disconnect_world(world);
+  return FALSE;
+}
+#endif
+
+
+static
+gboolean
+connected_to(World *world)
+{
+  gchar *status_text;
+
+#ifdef HAVE_LIBGNUTLS
+  if (world->use_tls) {
+#ifndef __MINGW32__
+  int    flags;
+#else /* defined __MINGW32__ */
+  u_long flags;
+#endif
+    int ret;
+
+#ifndef __MINGW32__
+    flags = fcntl(world->sock, F_GETFL);
+    flags &= ~O_NONBLOCK;
+    fcntl(world->sock, F_SETFL, flags);
+#else /* defined __MINGW32__ */
+    flags = 0;
+    ioctlsocket(sock, FIONBIO, &flags);
+#endif
+    gnutls_transport_set_ptr(world->tls_session,
+#ifndef __x86_64      /* This is to remove a warning about pointer sizes */
+                               (gnutls_transport_ptr) world->sock);
+#else
+                               (gnutls_transport_ptr) (long) world->sock);
+#endif
+    /* Perform the TLS handshake */
+    ret = gnutls_handshake(world->tls_session);
+    if (ret < 0) {
+      gnutls_deinit(world->tls_session);
+      world->errmsg = g_strdup_printf(_("SSL Handshake failed: %s"),
+                                      gnutls_strerror(ret));
+      disconnect_world(world);
+      return TRUE;
+    }
+  }
+#endif
+
+  status_text = g_strdup_printf(_("Connected%s to host %s (%s) port %s."),
+                                world->use_tls ? _(" with SSL") : "",
+                                world->host, world->host_ip, world->port);
+  ansitextview_append_string_with_fgcolor(world->gui, status_text,
+                                          globalPrefs.idxInfoMsgColor);
+  ansitextview_append_string(world->gui, "\n");
+  gtk_label_set_text(world->gui->lblStatus, status_text);
+  g_free(status_text);
+
+  perform_auto_logon(world);
+  execute_hook(world, "OnConnect", NULL);
+  if (world->log_autostart) {
+    gchar *errmsg = NULL;
+
+    stop_log(world);
+    if (start_log(world, &errmsg)) {
+      ansitextview_append_stringf(world->gui,
+                                  _("Logging to file %s.\n"),
+                                  world->log_actual_file);
+    } else {
+      ansitextview_append_string_nl(world->gui, errmsg);
+      g_free(errmsg);
+    }
+  }
+
+  world->connected         = TRUE;
+  world->connecting        = FALSE;
+  world->connected_time    = time(NULL);
+  world->last_command_time = time(NULL);
+
+  /* We now read the socket */
+  g_source_remove(world->io_watch_id);
+  world->io_watch_id = g_io_add_watch_full(world->iochan,
+                                           G_PRIORITY_HIGH,
+                                           G_IO_IN | G_IO_HUP | G_IO_ERR,
+                                           data_ready_cb,
+                                           world,
+                                           NULL);
+
+  /* No longer needed */
+  freeaddrinfo(world->addrinfo);
+  world->addrinfo = NULL;
+
+  return TRUE;
+}
+
 
 gboolean
 disconnect_world(gpointer data)
@@ -105,10 +581,16 @@ disconnect_world(gpointer data)
 
   world = (World *) data;
 
-  if (!world->connected) {
+  if (!world->connected && !world->connecting) {
     return FALSE;
   }
 
+  /* In case we are still connecting and it has not been free'd */
+  if (world->addrinfo) {
+    freeaddrinfo(world->addrinfo);
+    world->addrinfo = NULL;
+  }
+
   msg = close_connection(world);
 
   execute_hook(world, "OnDisconnect", NULL);
@@ -135,6 +617,11 @@ disconnect_world(gpointer data)
 gboolean
 reconnect_world(gpointer data)
 {
+  World *world = (World *) data;
+
+  if (world->connected || world->connecting)
+    g_free(close_connection(world));
+
   /* This function is called as an idle function, so it must return FALSE. */
   disconnected_decide_action((World *) data, RESP_RECONNECT);
   return FALSE;
@@ -193,16 +680,15 @@ gchar *
 close_connection(World *world)
 {
   /* Note: this function should only be called if the world is
-     connected. */
+     connected or connecting. */
   gchar *msg;
 
   execute_hook(world, "OnCloseConnected", NULL);
 
   flush_buffer(world, 2);
-  if (world->err_no) {
-    msg = g_strdup_printf(_("Disconnected from host %s: %s."),
-                          world->host, g_strerror(world->err_no));
-    world->err_no = 0;
+  if (world->errmsg) {
+    msg = world->errmsg;
+    world->errmsg = NULL;
   } else {
     msg = g_strdup_printf(_("Disconnected from host %s."), world->host);
   }
@@ -210,17 +696,24 @@ close_connection(World *world)
                                           globalPrefs.idxInfoMsgColor);
   ansitextview_append_string(world->gui, "\n");
 
+#ifndef __MINGW32__
   close(world->sock);
+#else /* defined __MINGW32__ */
+  closesocket(world->sock);
+#endif
   world->connected = FALSE;
+  world->connecting = FALSE;
 #ifdef USE_LIBGNUTLS
   if (world->use_tls) {
     gnutls_deinit(world->tls_session);
   }
 #endif
 
-  g_io_channel_unref(world->iochan);
+  if (world->iochan) {
+    g_io_channel_unref(world->iochan);
+    g_source_remove(world->io_watch_id);
+  }
 
-  g_source_remove(world->io_watch_id);
   if (world->timeout_id) {
     g_source_remove(world->timeout_id);
     world->timeout_id = 0;
@@ -234,6 +727,7 @@ close_connection(World *world)
 }
 
 
+static
 gboolean
 data_ready_cb(GIOChannel   *iochan,
               GIOCondition  cond,
@@ -257,10 +751,14 @@ data_ready_cb(GIOChannel   *iochan,
                   world->inbuffer,
                   MAX_BUFFER - world->pbuf_size);
   if (nread <= 0) {      /* Connection closed */
-    if (nread == -1) {
-      world->err_no = errno;
-    } else {
-      world->err_no = 0;
+    if (nread != 0) {    /* Error */
+#ifndef __MINGW32__
+      world->errmsg = g_strdup_printf(_("Disconnected from host %s: %s."),
+                                      world->host, g_strerror(errno));
+#else /* defined __MINGW32__ */
+      world->errmsg = g_strdup_printf(_("Disconnected from host %s: error #%d."),
+                                      world->host, WSAGetLastError());
+#endif
     }
     disconnect_world(world);
     return FALSE;
@@ -767,7 +1265,7 @@ start_mccp(World *world, int pos, int end)
       world->zstream->next_out  = world->dbuffer;
       world->zstream->avail_out = MAX_BUFFER;
 
-      status_text = g_strdup_printf(_("Connected%s to host %s (%s) port %d, MCCP version %d enabled."),
+      status_text = g_strdup_printf(_("Connected%s to host %s (%s) port %s, MCCP version %d enabled."),
                                     world->use_tls ? _(" with SSL") : "",
                                     world->host, world->host_ip, world->port,
                                     world->mccp_ver);
@@ -792,7 +1290,7 @@ end_mccp(World *world)
   g_free(world->zstream);
   world->zstream = NULL;
 
-  status_text = g_strdup_printf(_("Connected%s to host %s (%s) port %d."),
+  status_text = g_strdup_printf(_("Connected%s to host %s (%s) port %s."),
                                 world->use_tls ? _(" with SSL") : "",
                                 world->host, world->host_ip, world->port);
   gtk_label_set_text(world->gui->lblStatus, status_text);
@@ -1002,7 +1500,7 @@ send_naws_size(World *world)
      values. */
   guchar   buffer[3 + 8 + 2];
   int      bsize;
-  uint16_t size;
+  guint16  size;
   guint    cols;
   guint    rows;
 
@@ -1013,7 +1511,7 @@ send_naws_size(World *world)
   buffer[2] = TELNET_OPT_NAWS;
   bsize = 3;
 
-  size = htons((uint16_t) cols);
+  size = htons((guint16) cols);
   memcpy(buffer + bsize, &size, sizeof(size));
   /* Escape IAC */
   if (buffer[bsize] == TELNET_IAC) {
@@ -1029,7 +1527,7 @@ send_naws_size(World *world)
   }
   ++bsize;
 
-  size = htons((uint16_t) rows);
+  size = htons((guint16) rows);
   memcpy(buffer + bsize, &size, sizeof(size));
   /* Escape IAC */
   if (buffer[bsize] == TELNET_IAC) {
