@@ -1,4 +1,4 @@
-/* $Id: perlscript.c,v 1.117 2006/08/10 14:36:33 ekalin Exp $ */
+/* $Id: perlscript.c,v 1.123 2007/11/24 19:14:36 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -178,7 +178,13 @@ create_world_var(World *world)
 void
 destruct_perl_script(PerlInterpreter *perl_interpreter)
 {
+#ifndef __MINGW32__
+  /* For some reason, this function crashes under Windows when a World
+     is closed and there is another one open. I don't know how
+     important it is to call this function, but at least the one that
+     frees the resources is called. */
   perl_destruct(perl_interpreter);
+#endif
   perl_free(perl_interpreter);
 }
 
@@ -221,7 +227,7 @@ perl_match(PerlInterpreter     *interpreter,
   /* Create the command to run */
   command = newSVpvf("$triggerline =~ tr/\\n\\r//d;"
                      "my $target = %d;"
-                     "my @__M = ($triggerline =~ m<$_re>);"
+                     "my @__M = ($triggerline =~ m\x05$_re\x05);"
                      "my $match = scalar @__M;"
                      "if ($match) {"
                      "  @_ = @__M;"
@@ -282,7 +288,7 @@ perl_substitute(PerlInterpreter *interpreter,
   re = get_sv("_re", TRUE);
   sv_setsv(re, pattern_re);
   /* Create the command to run */
-  command = newSVpvf("$__aliasmatched = ($aliascmd =~ s<$_re><%s>%s);",
+  command = newSVpvf("$__aliasmatched = ($aliascmd =~ s\x05$_re\x05%s\x05%s);",
                      substitution,
                      perl_eval ? "e" : "");
 
@@ -432,7 +438,6 @@ XS_dc(pTHX_ CV *perlcv)
      can destroy the Perl interpreter if another world is loaded, and
      this would cause a segfault in the function that evaluates
      Perl commands. */
-  world->err_no = 0;
   g_idle_add(disconnect_world, world);
 
   XSRETURN_EMPTY;
@@ -449,7 +454,6 @@ XS_reconnect(pTHX_ CV *perlcv)
   FETCHWORLD;
 
   if (world->connected) {
-    world->err_no = 0;
     g_free(close_connection(world));
   }
 
@@ -687,6 +691,34 @@ XS_getname(pTHX_ CV *perlcv)
 }
 
 
+static
+void
+XS_getcharacter(pTHX_ CV *perlcv)
+{
+  GtkTreeIter  iter;
+  gchar       *character = NULL;
+  FETCHWORLDVARS;
+  dXSARGS;
+
+  FETCHWORLD;
+
+  if (world->character_used == -1 || world->connection_style == NONE) {
+    XSRETURN_UNDEF;
+  }
+
+  gtk_tree_model_iter_nth_child(world->logon_characters,
+                                &iter, NULL,
+                                world->character_used);
+  gtk_tree_model_get(world->logon_characters, &iter,
+                     LOGON_CHAR, &character,
+                     -1);
+  XST_mPV(0, character);
+  g_free(character);
+
+  XSRETURN(1);
+}
+
+
 static
 void
 XS_getmainfont(pTHX_ CV *perlcv)
@@ -2426,6 +2458,37 @@ XS_maketemporary(pTHX_ CV *perlcv)
 }
 
 
+static
+void
+XS_ispermanent(pTHX_ CV *perlcv)
+{
+  char    *var;
+  GSList  *list;
+  FETCHWORLDVARSFULL;
+  dXSARGS;
+
+  FETCHWORLDFULL;
+
+  if (items < 2) {
+    warn(_("%s: Too few arguments"), "ispermanent");
+    XSRETURN_EMPTY;
+  }
+
+  var = SvPV_nolen(ST(1));
+
+  list = g_slist_find_custom(world->permanent_variables,
+                             var,
+                             (GCompareFunc) strcmp);
+  if (list) {
+    XST_mIV(0, TRUE);
+  } else {
+    XST_mIV(0, FALSE);
+  }
+
+  XSRETURN(1);
+}
+
+
 static
 void
 XS_listpermanent(pTHX_ CV *perlcv)
@@ -2636,24 +2699,29 @@ static
 void
 XS_logfile(pTHX_ CV *perlcv)
 {
-  char *timestr;
   char *tmp;
-  int   has_timeformat = 0;
+  char *errmsg = NULL;
   FETCHWORLDVARS;
   dXSARGS;
 
   FETCHWORLD;
 
+  /* This will be overwritten if there are two arguments. */
+  world->log_add_time = FALSE;
+
+  /* If logging is started, it must be finished to start again
+     in the new file and/or with the new format. */
+  if (world->log_file) {
+    stop_log(world);
+  }
+
   switch (items) {
   case 1:
-    if (world->log_file) {
-      close_log_file(world, TRUE);
-    }
     ansitextview_append_string_nl(world->gui, _("Logging stopped."));
     break;
 
   case 3:
-    has_timeformat = 1;
+    world->log_add_time = TRUE;
     tmp = SvPV_nolen(ST(2));
     g_free(world->log_timeformat);
     world->log_timeformat = g_strdup(tmp);
@@ -2661,26 +2729,22 @@ XS_logfile(pTHX_ CV *perlcv)
 
   case 2:
     tmp = SvPV_nolen(ST(1));
+    g_free(world->log_file_name);
+    world->log_file_name = g_strdup(tmp);
 
-    timestr = close_log_file(world, has_timeformat);
-
-    if ((world->log_file = fopen(tmp, "a")) != NULL) {
-      /* Use unbuffered output */
-      setbuf(world->log_file, NULL);
-      world->log_file_name = g_strdup(tmp);
-
-      fprintf(world->log_file, "*** LOG STARTED AT %s ***\n",
-              timestr);
-
+    if (start_log(world, &errmsg)) {
       ansitextview_append_stringf(world->gui,
                                   _("Logging to file %s.\n"),
-                                  world->log_file_name);
+                                  world->log_actual_file);
     } else {
-      warn(_("Could no open log file."));
+      ansitextview_append_string_nl(world->gui, errmsg);
+      g_free(errmsg);
     }
     break;
   }
 
+  fill_we_logging_tab(world, NULL);
+
   XSRETURN_EMPTY;
 }
 
@@ -3003,6 +3067,7 @@ xs_init()
   newXS("getworld",            XS_getworld,     "");
 
   newXS("World::getname",      XS_getname,      "World");
+  newXS("World::getcharacter", XS_getcharacter, "World");
   newXS("World::getmainfont",  XS_getmainfont,  "World");
   newXS("World::getentryfont", XS_getentryfont, "World");
   newXS("World::commandecho",  XS_commandecho,  "World");
@@ -3044,6 +3109,7 @@ xs_init()
 
   newXS("World::makepermanent", XS_makepermanent, "World");
   newXS("World::maketemporary", XS_maketemporary, "World");
+  newXS("World::ispermanent",   XS_ispermanent,   "World");
   newXS("World::listpermanent", XS_listpermanent, "World");
 
   newXS("World::hook",      XS_hook,      "World");
