@@ -1,4 +1,4 @@
-/* $Id: timers.c,v 1.14 2005/11/30 13:10:21 ekalin Exp $ */
+/* $Id: timers.c,v 1.15 2007/08/20 12:35:53 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -153,6 +153,8 @@ void
 list_timers(World *world, Plugin *plugin)
 {
   int     i;
+  guint   interval_width;
+  guint   repeat_width;
   guint   total_width;
   guint   field_width;
   GSList *timerptr;
@@ -165,10 +167,50 @@ list_timers(World *world, Plugin *plugin)
   if (total_width < 29) {
     total_width = 29;
   }
-  field_width = total_width - 23;
 
-  ansitextview_append_string_nl(world->gui,_("Num Int Count Ena Temp Action"));
-  ansitextview_append_string(world->gui, "--- --- ----- --- ---- ");
+  /* Calculate maximum size necessary for the interval and repeat
+     count fields */
+  interval_width = 3;
+  repeat_width   = 5;
+  timerptr = world->timers;
+  while (timerptr) {
+    guint this_int_width;
+    guint this_rep_width;
+
+    timer = (Timer *) timerptr->data;
+
+    this_int_width = (guint) log10(timer->interval) + 1;
+    if (this_int_width > interval_width) {
+      interval_width = this_int_width;
+    }
+    this_rep_width = (guint) log10(timer->count) + 1;
+    if (this_rep_width > repeat_width) {
+      repeat_width = this_rep_width;
+    }
+
+    timerptr = timerptr->next;
+  }
+
+  field_width = total_width - 13 - interval_width - 1 - repeat_width - 1;
+
+  ansitextview_append_string(world->gui, _("Num Ena Temp Int "));
+  for (i = 0; i < interval_width - 3; ++i) {
+    ansitextview_append_string(world->gui, " ");
+  }
+  ansitextview_append_string(world->gui, _("Count "));
+  for (i = 0; i < repeat_width - 5; ++i) {
+    ansitextview_append_string(world->gui, " ");
+  }
+  ansitextview_append_string_nl(world->gui, _("Action"));
+  ansitextview_append_string(world->gui, "--- --- ---- ");
+  for (i = 0; i < interval_width; ++i) {
+    ansitextview_append_string(world->gui, "-");
+  }
+  ansitextview_append_string(world->gui, " ");
+  for (i = 0; i < repeat_width; ++i) {
+    ansitextview_append_string(world->gui, "-");
+  }
+  ansitextview_append_string(world->gui, " ");
   for (i = 0; i < field_width; ++i) {
     ansitextview_append_string(world->gui, "-");
   }
@@ -180,12 +222,12 @@ list_timers(World *world, Plugin *plugin)
     timer = (Timer *) timerptr->data;
 
     if (timer->owner_plugin == plugin) {
-      ansitextview_append_stringf(world->gui,"%3d %3d %5d %3.3s %4.4s %-*.*s\n",
+      ansitextview_append_stringf(world->gui,"%3d %3.3s %4.4s %*u %*d %-*.*s\n",
                                   i,
-                                  timer->interval,
-                                  timer->count,
                                   timer->enabled ? _("y") : _("n"),
                                   timer->temporary ? _("y") : _("n"),
+                                  interval_width, timer->interval,
+                                  repeat_width, timer->count,
                                   field_width, field_width, timer->action);
     }
 
