@@ -1,4 +1,4 @@
-/* $Id: we_general.c,v 1.9 2006/06/22 22:03:39 ekalin Exp $ */
+/* $Id: we_general.c,v 1.13 2007/11/25 18:37:06 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -65,7 +65,6 @@ get_we_general_tab(GladeXML *gladexml)
 void
 fill_we_general_tab(World *world, GladeXML *gladexml)
 {
-  gchar             *tmp;
   GtkWidget         *txtName;
   GtkWidget         *txtHost;
   GtkWidget         *txtPort;
@@ -102,9 +101,7 @@ fill_we_general_tab(World *world, GladeXML *gladexml)
   if (world->host) {
     gtk_entry_set_text(GTK_ENTRY(txtHost), world->host);
   }
-  tmp = g_strdup_printf("%d", world->port);
-  gtk_entry_set_text(GTK_ENTRY(txtPort), tmp);
-  g_free(tmp);
+  gtk_entry_set_text(GTK_ENTRY(txtPort), world->port);
 #ifdef HAVE_LIBGNUTLS
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkTLS), world->use_tls);
 #else
@@ -157,7 +154,6 @@ void
 update_we_general_parameters(World *world)
 {
   const gchar *newval;
-  int          newport;
   GladeXML    *gladexml;
   GtkWidget   *txtName;
   GtkWidget   *txtHost;
@@ -188,9 +184,9 @@ update_we_general_parameters(World *world)
   }
 
   newval = gtk_entry_get_text(GTK_ENTRY(txtPort));
-  newport = atoi(newval);
-  if (world->port != newport) {
-    world->port = newport;
+  if (!world->port || strcmp(world->port, newval) != 0) {
+    g_free(world->port);
+    world->port = g_strdup(newval);
   }
 
   world->connection_style = gtk_combo_box_get_active(GTK_COMBO_BOX(cmbConnStyle));
@@ -305,6 +301,7 @@ edit_char(World *world, GtkTreeModel *model, GtkTreeIter *iter)
                                                 GTK_MESSAGE_ERROR,
                                                 GTK_BUTTONS_OK,
                                                 _("You must specify the name."));
+        gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Warning"));
         gtk_dialog_run(GTK_DIALOG(dlg));
         gtk_widget_destroy(dlg);
         continue;
@@ -317,6 +314,7 @@ edit_char(World *world, GtkTreeModel *model, GtkTreeIter *iter)
                                                 GTK_MESSAGE_ERROR,
                                                 GTK_BUTTONS_OK,
                                                 _("You must specify the password."));
+        gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Warning"));
         gtk_dialog_run(GTK_DIALOG(dlg));
         gtk_widget_destroy(dlg);
         continue;
@@ -356,30 +354,10 @@ we_general_delete_char_cb(GtkWidget *widget, gpointer data)
   selection = gtk_tree_view_get_selection(view);
   if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
     GtkWidget *msgdlg;
-    GtkWidget *dlgbutton;
-    GtkWidget *dlgbuttonimage;
-
-    msgdlg = gtk_message_dialog_new(GTK_WINDOW(world->dlgEditWorld),
-                                    GTK_DIALOG_MODAL,
-                                    GTK_MESSAGE_QUESTION,
-                                    GTK_BUTTONS_NONE,
-                                    _("Do you really want to delete this character information?"));
-    dlgbutton = gtk_button_new_with_label(_("Do not delete"));
-    dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_NO,
-                                              GTK_ICON_SIZE_MENU);
-    gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-    gtk_widget_show_all(dlgbutton);
-    gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                                 dlgbutton, GTK_RESPONSE_NO);
-
-    dlgbutton = gtk_button_new_with_label(_("Delete"));
-    dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_DELETE,
-                                              GTK_ICON_SIZE_MENU);
-    gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-    gtk_widget_show_all(dlgbutton);
-    gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                                 dlgbutton, GTK_RESPONSE_YES);
 
+    msgdlg = we_guied_confirm_delete_dialog_new(GTK_WINDOW(world->dlgEditWorld),
+                                                1,
+                                                _("character"), "");
 
     if (!world->confirm_delete
         || gtk_dialog_run(GTK_DIALOG(msgdlg)) == GTK_RESPONSE_YES) {
@@ -396,7 +374,7 @@ we_general_view_keypress_cb(GtkWidget   *widget,
                             GdkEventKey *evt,
                             gpointer     data)
 {
-  if (evt->keyval == GDK_Delete) {
+  if (evt->keyval == GDK_Delete || evt->keyval == GDK_KP_Delete) {
     we_general_delete_char_cb(widget, data);
     return TRUE;
   }
