@@ -1,4 +1,4 @@
-/* $Id: we_generic_guied.c,v 1.7 2006/08/10 14:36:33 ekalin Exp $ */
+/* $Id: we_generic_guied.c,v 1.9 2007/11/25 18:37:06 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -121,6 +121,7 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
                                             _("Could not open file '%s': %s"),
                                             file,
                                             g_strerror(errno));
+    gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Error"));
     gtk_dialog_run(GTK_DIALOG(dlg));
     gtk_widget_destroy(dlg);
     return;
@@ -376,6 +377,7 @@ we_guied_do_import(World *world, const gchar *filename)
                                                _("Could not import file '%s': %s"),
                                                filename,
                                                error->message);
+    gtk_window_set_title(GTK_WINDOW(msgdlg), _("KildClient Error"));
     gtk_dialog_run(GTK_DIALOG(msgdlg));
     gtk_widget_destroy(msgdlg);
     free_world(newworld);
@@ -909,3 +911,50 @@ we_guied_selection_changed_cb(GtkTreeSelection *selection,
   gtk_widget_set_sensitive(btnDelete, n >= 1);
   gtk_widget_set_sensitive(btnExport, n >= 1);
 }
+
+
+GtkWidget *
+we_guied_confirm_delete_dialog_new(GtkWindow   *parent,
+                                   int          n,
+                                   const gchar *singular,
+                                   const gchar *plural)
+{
+  GtkWidget *msgdlg;
+  GtkWidget *dlgbutton;
+  GtkWidget *dlgbuttonimage;
+
+  if (n == 1) {
+    msgdlg = gtk_message_dialog_new(parent,
+                                    GTK_DIALOG_MODAL,
+                                    GTK_MESSAGE_QUESTION,
+                                    GTK_BUTTONS_NONE,
+                                    _("Do you really want to delete the %s?"),
+                                    singular);
+  } else {
+    msgdlg = gtk_message_dialog_new(parent,
+                                    GTK_DIALOG_MODAL,
+                                    GTK_MESSAGE_QUESTION,
+                                    GTK_BUTTONS_NONE,
+                                    _("Do you really want to delete %d %s?"),
+                                    n,
+                                    plural);
+  }
+  gtk_window_set_title(GTK_WINDOW(msgdlg), _("Confirm deletion"));
+  dlgbutton = gtk_button_new_with_label(_("Do not delete"));
+  dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_NO,
+                                            GTK_ICON_SIZE_MENU);
+  gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
+  gtk_widget_show_all(dlgbutton);
+  gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
+                               dlgbutton, GTK_RESPONSE_NO);
+
+  dlgbutton = gtk_button_new_with_label(_("Delete"));
+  dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_DELETE,
+                                            GTK_ICON_SIZE_MENU);
+  gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
+  gtk_widget_show_all(dlgbutton);
+  gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
+                               dlgbutton, GTK_RESPONSE_YES);
+
+  return msgdlg;
+}
