@@ -1,4 +1,4 @@
-/* $Id: we_macros.c,v 1.10 2006/06/22 22:03:39 ekalin Exp $ */
+/* $Id: we_macros.c,v 1.13 2007/11/25 18:37:06 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -327,42 +327,12 @@ we_macro_delete_cb(GtkButton *button, gpointer data)
     Macro     *macro;
     GSList    *macroitem;
     GtkWidget *msgdlg;
-    GtkWidget *dlgbutton;
-    GtkWidget *dlgbuttonimage;
     gint       n;
 
     n = gtk_tree_selection_count_selected_rows(selection);
-    if (n == 1) {
-      msgdlg = gtk_message_dialog_new(GTK_WINDOW(world->dlgEditWorld),
-                                      GTK_DIALOG_MODAL,
-                                      GTK_MESSAGE_QUESTION,
-                                      GTK_BUTTONS_NONE,
-                                      _("Do you really want to delete the %s?"),
-                                      _("macro"));
-    } else {
-      msgdlg = gtk_message_dialog_new(GTK_WINDOW(world->dlgEditWorld),
-                                      GTK_DIALOG_MODAL,
-                                      GTK_MESSAGE_QUESTION,
-                                      GTK_BUTTONS_NONE,
-                                      _("Do you really want to delete %d %s?"),
-                                      n,
-                                      _("macros"));
-    }
-    dlgbutton = gtk_button_new_with_label(_("Do not delete"));
-    dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_NO,
-                                              GTK_ICON_SIZE_MENU);
-    gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-    gtk_widget_show_all(dlgbutton);
-    gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                                 dlgbutton, GTK_RESPONSE_NO);
-
-    dlgbutton = gtk_button_new_with_label(_("Delete"));
-    dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_DELETE,
-                                              GTK_ICON_SIZE_MENU);
-    gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-    gtk_widget_show_all(dlgbutton);
-    gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                                 dlgbutton, GTK_RESPONSE_YES);
+    msgdlg = we_guied_confirm_delete_dialog_new(GTK_WINDOW(world->dlgEditWorld),
+                                                n,
+                                                _("macro"), _("macros"));
 
     if (!world->confirm_delete
         || gtk_dialog_run(GTK_DIALOG(msgdlg)) == GTK_RESPONSE_YES) {
@@ -482,6 +452,7 @@ edit_macro(World *world, Macro *macro)
                                                 GTK_MESSAGE_ERROR,
                                                 GTK_BUTTONS_OK,
                                                 _("You must specify the key."));
+        gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Warning"));
         gtk_dialog_run(GTK_DIALOG(dlg));
         gtk_widget_destroy(dlg);
         continue;
@@ -494,6 +465,7 @@ edit_macro(World *world, Macro *macro)
                                                 GTK_MESSAGE_ERROR,
                                                 GTK_BUTTONS_OK,
                                                 _("Invalid key code."));
+        gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Warning"));
         gtk_dialog_run(GTK_DIALOG(dlg));
         gtk_widget_destroy(dlg);
         continue;
@@ -506,6 +478,7 @@ edit_macro(World *world, Macro *macro)
                                                 GTK_MESSAGE_ERROR,
                                                 GTK_BUTTONS_OK,
                                                 _("You must specify the action."));
+        gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Warning"));
         gtk_dialog_run(GTK_DIALOG(dlg));
         gtk_widget_destroy(dlg);
         continue;
@@ -549,7 +522,7 @@ we_macro_view_keypress_cb(GtkWidget   *widget,
                           GdkEventKey *evt,
                           gpointer     data)
 {
-  if (evt->keyval == GDK_Delete) {
+  if (evt->keyval == GDK_Delete || evt->keyval == GDK_KP_Delete) {
     we_macro_delete_cb(NULL, widget);
     return TRUE;
   }
