@@ -1,4 +1,4 @@
-/* $Id: we_plugins.c,v 1.8 2006/06/22 22:03:39 ekalin Exp $ */
+/* $Id: we_plugins.c,v 1.10 2007/11/25 18:19:35 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -228,6 +228,7 @@ we_splugin_add_cb(GtkButton *button, gpointer data)
                                               GTK_MESSAGE_ERROR,
                                               GTK_BUTTONS_OK,
                                               error->message);
+      gtk_window_set_title(GTK_WINDOW(dlg), _("Error Loading Plugin"));
       gtk_dialog_run(GTK_DIALOG(dlg));
       gtk_widget_destroy(dlg);
       g_error_free(error);
@@ -294,7 +295,7 @@ we_splugin_view_keypress_cb(GtkWidget   *widget,
                             GdkEventKey *evt,
                             gpointer     data)
 {
-  if (evt->keyval == GDK_Delete) {
+  if (evt->keyval == GDK_Delete || evt->keyval == GDK_KP_Delete) {
     we_splugin_remove_cb(NULL, widget);
     return TRUE;
   }
@@ -487,6 +488,7 @@ we_lplugin_load_cb(GtkButton *button, gpointer data)
                                               GTK_MESSAGE_ERROR,
                                               GTK_BUTTONS_OK,
                                               error->message);
+      gtk_window_set_title(GTK_WINDOW(dlg), _("Error Loading Plugin"));
       gtk_dialog_run(GTK_DIALOG(dlg));
       gtk_widget_destroy(dlg);
       g_error_free(error);
@@ -528,6 +530,7 @@ we_lplugin_help_cb(GtkButton *button, gpointer data)
                                       GTK_MESSAGE_INFO,
                                       GTK_BUTTONS_OK,
                                       _("The help is displayed in main window."));
+      gtk_window_set_title(GTK_WINDOW(msgdlg), _("KildClient Information"));
       chk = gtk_check_button_new_with_mnemonic(_("_Do not display this message again"));
       gtk_box_pack_start(GTK_BOX(GTK_DIALOG(msgdlg)->vbox), chk,
                          FALSE, FALSE, 4);
