@@ -1,4 +1,4 @@
-/* $Id: we_triggers.c,v 1.14 2006/08/10 14:36:33 ekalin Exp $ */
+/* $Id: we_triggers.c,v 1.17 2007/11/25 18:37:05 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -451,43 +451,12 @@ we_trigger_delete_cb(GtkButton *button, gpointer data)
     Trigger   *trigger;
     GSList    *triggeritem;
     GtkWidget *msgdlg;
-    GtkWidget *dlgbutton;
-    GtkWidget *dlgbuttonimage;
     gint       n;
 
     n = gtk_tree_selection_count_selected_rows(selection);
-    if (n == 1) {
-      msgdlg = gtk_message_dialog_new(GTK_WINDOW(world->dlgEditWorld),
-                                      GTK_DIALOG_MODAL,
-                                      GTK_MESSAGE_QUESTION,
-                                      GTK_BUTTONS_NONE,
-                                      _("Do you really want to delete the %s?"),
-                                      _("trigger"));
-    } else {
-      msgdlg = gtk_message_dialog_new(GTK_WINDOW(world->dlgEditWorld),
-                                      GTK_DIALOG_MODAL,
-                                      GTK_MESSAGE_QUESTION,
-                                      GTK_BUTTONS_NONE,
-                                      _("Do you really want to delete %d %s?"),
-                                      n,
-                                      _("triggers"));
-    }
-    dlgbutton = gtk_button_new_with_label(_("Do not delete"));
-    dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_NO,
-                                              GTK_ICON_SIZE_MENU);
-    gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-    gtk_widget_show_all(dlgbutton);
-    gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                                 dlgbutton, GTK_RESPONSE_NO);
-
-    dlgbutton = gtk_button_new_with_label(_("Delete"));
-    dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_DELETE,
-                                              GTK_ICON_SIZE_MENU);
-    gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-    gtk_widget_show_all(dlgbutton);
-    gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                                 dlgbutton, GTK_RESPONSE_YES);
-
+    msgdlg = we_guied_confirm_delete_dialog_new(GTK_WINDOW(world->dlgEditWorld),
+                                                n,
+                                                _("trigger"), _("triggers"));
     if (!world->confirm_delete
         || gtk_dialog_run(GTK_DIALOG(msgdlg)) == GTK_RESPONSE_YES) {
       GList *listiter = g_list_last(selected);
@@ -637,6 +606,7 @@ edit_trigger(World *world, Trigger *trigger)
                                                 GTK_MESSAGE_ERROR,
                                                 GTK_BUTTONS_OK,
                                                 _("You must specify the pattern."));
+        gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Warning"));
         gtk_dialog_run(GTK_DIALOG(dlg));
         gtk_widget_destroy(dlg);
         continue;
@@ -650,6 +620,7 @@ edit_trigger(World *world, Trigger *trigger)
                                                 GTK_MESSAGE_ERROR,
                                                 GTK_BUTTONS_OK,
                                                 _("You must specify the action, unless the trigger is a gag or changes the style."));
+        gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Warning"));
         gtk_dialog_run(GTK_DIALOG(dlg));
         gtk_widget_destroy(dlg);
         continue;
@@ -824,7 +795,7 @@ we_trigger_view_keypress_cb(GtkWidget   *widget,
                             GdkEventKey *evt,
                             gpointer     data)
 {
-  if (evt->keyval == GDK_Delete) {
+  if (evt->keyval == GDK_Delete || evt->keyval == GDK_KP_Delete) {
     we_trigger_delete_cb(NULL, widget);
     return TRUE;
   }
