@@ -1,4 +1,4 @@
-/* $Id: we_vars.c,v 1.10 2006/06/22 22:03:39 ekalin Exp $ */
+/* $Id: we_vars.c,v 1.12 2007/11/25 18:19:35 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -275,6 +275,7 @@ we_var_delete_cb(GtkButton *button, gpointer data)
                                       _("Do you really want to make %d variables temporary?"),
                                       n);
     }
+    gtk_window_set_title(GTK_WINDOW(msgdlg), _("Confirm deletion"));
     dlgbutton = gtk_button_new_with_label(_("Keep permanent"));
     dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_NO,
                                               GTK_ICON_SIZE_MENU);
@@ -383,6 +384,7 @@ edit_var(World *world, const gchar *var)
                                                 GTK_MESSAGE_ERROR,
                                                 GTK_BUTTONS_OK,
                                                 _("You must specify a variable name."));
+        gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Warning"));
         gtk_dialog_run(GTK_DIALOG(dlg));
         gtk_widget_destroy(dlg);
         continue;
@@ -394,6 +396,7 @@ edit_var(World *world, const gchar *var)
                                                 GTK_MESSAGE_ERROR,
                                                 GTK_BUTTONS_OK,
                                                 _("Only scalar ($), array (@) and hash (%%) variables can be made permanent."));
+        gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Warning"));
         gtk_dialog_run(GTK_DIALOG(dlg));
         gtk_widget_destroy(dlg);
         continue;
@@ -413,6 +416,7 @@ edit_var(World *world, const gchar *var)
                                                 GTK_BUTTONS_OK,
                                                 _("%s is already permanent."),
                                                 givenname);
+        gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Warning"));
         gtk_dialog_run(GTK_DIALOG(dlg));
         gtk_widget_destroy(dlg);
         continue;
@@ -437,7 +441,7 @@ we_var_view_keypress_cb(GtkWidget   *widget,
                         GdkEventKey *evt,
                         gpointer     data)
 {
-  if (evt->keyval == GDK_Delete) {
+  if (evt->keyval == GDK_Delete || evt->keyval == GDK_KP_Delete) {
     we_var_delete_cb(NULL, widget);
     return TRUE;
   }
