@@ -1,4 +1,4 @@
-/* $Id: worldeditor.c,v 1.30 2006/02/14 20:00:12 ekalin Exp $ */
+/* $Id: worldeditor.c,v 1.31 2007/11/24 19:14:36 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -115,6 +115,7 @@ world_editor_response_cb(GtkWidget *dialog, gint response, gpointer data)
     }
     /* Fonts and colors are updated automatically */
     update_we_input_parameters(world);
+    update_we_logging_parameters(world);
     if (world != default_world) {
       update_we_scripting_parameters(world);
     }
@@ -146,10 +147,11 @@ create_edit_world_dialog(World *world, GtkWindow *parent)
 
   gladexml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
                            "dlgEditWorld", NULL);
-  /* glade_xml_signal_autoconnect(gladexml); */
+  glade_xml_signal_autoconnect(gladexml);
 
   dlgEditWorld = glade_xml_get_widget(gladexml, "dlgEditWorld");
   gtk_dialog_set_default_response(GTK_DIALOG(dlgEditWorld), GTK_RESPONSE_OK);
+  g_object_set_data(G_OBJECT(gladexml), "dlgEditWorld", dlgEditWorld);
 
   notebook = glade_xml_get_widget(gladexml, "ntbkWEPanel");
   gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
@@ -252,6 +254,14 @@ create_model(GtkNotebook *notebook,
                      COLUMN_WIDGET, panel,
                      -1);
 
+  gtk_tree_store_append(model, &iter, NULL);
+  panel = get_we_logging_tab(gladexml, world);
+  fill_we_logging_tab(world, gladexml);
+  gtk_tree_store_set(model, &iter,
+                     COLUMN_NAME, _("Logging"),
+                     COLUMN_WIDGET, panel,
+                     -1);
+
   if (world != default_world) {
     gtk_tree_store_append(model, &iterAutomation, NULL);
     panel = new_descriptive_panel(_("Automation"),
