@@ -1,4 +1,4 @@
-/* $Id: worldgui.c,v 1.49 2007/04/22 20:33:10 ekalin Exp $ */
+/* $Id: worldgui.c,v 1.53 2007/11/18 20:10:39 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -74,6 +74,7 @@ static gboolean  completion_match_selected_cb(GtkEntryCompletion *widget,
                                               GtkTreeModel       *model,
                                               GtkTreeIter        *iter,
                                               gpointer            data);
+static void      tab_close_cb(GtkWidget *widget, gpointer data);
 static gboolean  textview_button_press_cb(GtkWidget      *text_view,
                                           GdkEventButton *event,
                                           gpointer        data);
@@ -142,6 +143,7 @@ world_gui_new(gboolean is_for_mud)
   gui = g_new0(WorldGUI, 1);
 
   gui->vbox = gtk_vbox_new(FALSE, 0);
+  g_object_set_data(G_OBJECT(gui->vbox), "gui", gui);
 
   gui->scrolled_win = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
   gtk_scrolled_window_set_policy(gui->scrolled_win,
@@ -325,7 +327,32 @@ world_gui_new(gboolean is_for_mud)
   gui->ta.sup_greek         = -1;
 
   if (is_for_mud) {   /* Special features */
+    GtkWidget *btnClose;
+    GtkRcStyle *rcstyle;
+    GtkWidget *iconCloseWidget;
+
+    /* Notebook tab */
+    gui->hboxTab = GTK_BOX(gtk_hbox_new(FALSE, 4));
     gui->lblNotebook = GTK_LABEL(gtk_label_new(_("No world")));
+    gtk_box_pack_start(gui->hboxTab, GTK_WIDGET(gui->lblNotebook),
+                       FALSE, FALSE, 0);
+
+    btnClose = gtk_button_new();
+    /* Make the button smaller. (Taken from gedit code) */
+    gtk_button_set_focus_on_click(GTK_BUTTON(btnClose), FALSE);
+    gtk_button_set_relief(GTK_BUTTON(btnClose), GTK_RELIEF_NONE);
+    rcstyle  = gtk_rc_style_new();
+    rcstyle->xthickness = rcstyle->ythickness = 0;
+    gtk_widget_modify_style(btnClose, rcstyle);
+    gtk_rc_style_unref(rcstyle);
+
+    iconCloseWidget = gtk_image_new_from_stock(GTK_STOCK_CLOSE,
+                                               GTK_ICON_SIZE_MENU);
+    g_signal_connect(G_OBJECT(btnClose), "clicked",
+                     G_CALLBACK(tab_close_cb), gui);
+    gtk_container_add(GTK_CONTAINER(btnClose), iconCloseWidget);
+    gtk_box_pack_start(GTK_BOX(gui->hboxTab), btnClose,
+                       FALSE, FALSE, 0);
 
     /* The color and font will be changed later to the colors and font
        of the opened world, but we set the background to black so that is
@@ -610,6 +637,7 @@ keypress_cb(GtkWidget   *widget,
 
   switch (evt->keyval) {
   case GDK_Left:
+  case GDK_KP_Left:
     if (evt->state & GDK_MOD1_MASK) { /* Alt+left */
       menu_perl_run_cb(NULL, "$world->prev");
       return TRUE;
@@ -617,12 +645,25 @@ keypress_cb(GtkWidget   *widget,
     return FALSE;
 
   case GDK_Right:
+  case GDK_KP_Right:
     if (evt->state & GDK_MOD1_MASK) { /* Alt+right */
       menu_perl_run_cb(NULL, "$world->next");
       return TRUE;
     }
     return FALSE;
 
+  case GDK_Tab:
+  case GDK_ISO_Left_Tab:
+    if (evt->state & GDK_CONTROL_MASK) {
+      if (evt->state & GDK_SHIFT_MASK) { /* Ctrl+Shift+Tab */
+        menu_perl_run_cb(NULL, "$world->prev");
+      } else {                           /* Ctrl+Tab */
+        menu_perl_run_cb(NULL, "$world->next");
+      }
+      return TRUE;
+    }
+    return FALSE;
+
   case '1': case '2': case '3': case '4': case '5':
   case '6': case '7': case '8': case '9':
     if (evt->state & GDK_MOD1_MASK) { /* Alt+digit */
@@ -636,18 +677,24 @@ keypress_cb(GtkWidget   *widget,
     return FALSE;
 
   case GDK_Up:
+  case GDK_KP_Up:
+    direction = 0;
+
   case GDK_Down:
+  case GDK_KP_Down:
     if (world->input_n_lines > 1 && world->cmd_position_changed) {
       return FALSE;
     }
-    prev_or_next_command(world, evt->keyval == GDK_Down);
+    prev_or_next_command(world, direction);
     return TRUE;
 
   case GDK_Page_Up:
+  case GDK_KP_Page_Up:
     direction = -1;
     /* Fall through */
 
   case GDK_Page_Down:
+  case GDK_KP_Page_Down:
     adjustment = gtk_scrolled_window_get_vadjustment(gui->scrolled_win);
     newval = gtk_adjustment_get_value(adjustment)
       + direction*adjustment->page_increment/2;
@@ -659,6 +706,7 @@ keypress_cb(GtkWidget   *widget,
     return TRUE;
 
   case GDK_End:
+  case GDK_KP_End:
     if (evt->state & GDK_CONTROL_MASK) {
       adjustment = gtk_scrolled_window_get_vadjustment(gui->scrolled_win);
       gtk_adjustment_set_value(adjustment,
@@ -806,6 +854,57 @@ completion_match_selected_cb(GtkEntryCompletion *widget,
 }
 
 
+static
+void
+tab_close_cb(GtkWidget *widget, gpointer data)
+{
+  WorldGUI *gui   = (WorldGUI *) data;
+  World    *world = gui->world;
+
+  if (world->connected) {
+    GtkWidget *msgdlg;
+    GtkWidget *dlgbutton;
+    GtkWidget *dlgbuttonimage;
+    gint       response;
+
+    msgdlg = gtk_message_dialog_new(GTK_WINDOW(wndMain),
+                                    GTK_DIALOG_MODAL,
+                                    GTK_MESSAGE_QUESTION,
+                                    GTK_BUTTONS_NONE,
+                                    _("Are you sure you want to close this world?"));
+    gtk_window_set_title(GTK_WINDOW(msgdlg), _("Really close?"));
+    /* Keep Open is RESPONSE_NO because of NO close. But we use
+       STOCK_YES to pass a positive image. */
+    dlgbutton = gtk_button_new_with_mnemonic(_("Keep _open"));
+    dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_YES,
+                                              GTK_ICON_SIZE_BUTTON);
+    gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
+    gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
+                                 dlgbutton, GTK_RESPONSE_NO);
+
+    /*    dlgbutton = gtk_button_new_with_label(_("Close"));
+    dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_CLOSE,
+                                              GTK_ICON_SIZE_BUTTON);
+    gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
+    gtk_widget_show_all(dlgbutton);*/
+    dlgbutton = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+    gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
+                                 dlgbutton, GTK_RESPONSE_YES);
+
+    gtk_widget_show_all(msgdlg);
+    response = gtk_dialog_run(GTK_DIALOG(msgdlg));
+    gtk_widget_destroy(msgdlg);
+
+    if (response != GTK_RESPONSE_YES) {
+      return;
+    }
+  }
+
+  remove_world(world, TRUE);
+}
+
+
+
 void
 clear_button_cb(GtkButton *button, gpointer data)
 {
