@@ -1,4 +1,4 @@
-/* $Id: worlds.c,v 1.91 2007/05/13 22:00:56 ekalin Exp $ */
+/* $Id: worlds.c,v 1.96 2007/11/25 18:19:35 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -59,9 +59,13 @@ typedef enum {
   KC_ANSICOLOR,
   KC_BACKGROUND,
   KC_SCROLL,
+  KC_LOG,
+  KC_LOGFILE,
+  KC_LOGTIMEFORMAT,
   KC_FLOOD_PREVENTION,
   KC_REPEAT_COMMANDS,
   KC_COMMAND_ECHO,
+  KC_STORE_COMMANDS,
   KC_COMMAND_HISTORY,
   KC_AUTO_COMPLETION,
   KC_COMMAND_SEPARATOR,
@@ -241,6 +245,7 @@ create_new_world(gboolean load_defaults)
 
   world->repeat_commands     = default_world->repeat_commands;
   world->cmd_echo            = default_world->cmd_echo;
+  world->store_commands      = default_world->store_commands;
   world->commands_to_save    = default_world->commands_to_save;
   world->input_n_lines       = default_world->input_n_lines;
   world->input_n_lines_saved = default_world->input_n_lines_saved;
@@ -259,6 +264,10 @@ create_new_world(gboolean load_defaults)
     = g_strdup(default_world->flood_prevention_command);
   world->repeat_count             = 1;
 
+  world->log_timeformat = g_strdup(default_world->log_timeformat);
+  world->log_add_time   = default_world->log_add_time;
+  world->log_autostart  = default_world->log_autostart;
+
   /* Initialize command history list. */
   world->cmd_list = GTK_TREE_MODEL(gtk_list_store_new(N_CMDLIST_COLUMNS,
                                                       G_TYPE_STRING,
@@ -309,6 +318,7 @@ create_default_world(void)
 
   default_world->repeat_commands     = FALSE;
   default_world->cmd_echo            = TRUE;
+  default_world->store_commands      = TRUE;
   default_world->commands_to_save    = 50;
   default_world->input_n_lines       = 1;
   default_world->input_n_lines_saved = 2;
@@ -323,6 +333,10 @@ create_default_world(void)
   default_world->max_equal_commands       = 20;
   default_world->flood_prevention_command = g_strdup("look");
 
+  default_world->log_timeformat = g_strdup(DEFAULT_LOGTIME_FORMAT);
+  default_world->log_add_time   = TRUE;
+  default_world->log_autostart  = FALSE;
+
   /* Now load everything from the file */
   kilddir = get_kildclient_directory_path();
   default_world_file = g_build_filename(kilddir, "defworld.cfg", NULL);
@@ -362,6 +376,7 @@ free_world(World *world)
   g_free(world->file);
   g_free(world->charset);
   g_free(world->host);
+  g_free(world->port);
   g_free(world->host_ip);
 
   g_free(world->pbuffer);
@@ -421,7 +436,10 @@ free_world(World *world)
     g_object_unref(world->cmd_list);
   }
 
-  close_log_file(world, FALSE);
+  stop_log(world);
+  g_free(world->log_file_name);
+  g_free(world->log_timeformat);
+  g_free(world->log_actual_file);
 
   if (world->connected_at_least_once) {
     destruct_perl_script(world->perl_interpreter);
@@ -531,17 +549,18 @@ load_world_from_file(const char *file,
 
   success = FALSE;
   while (1) {
-    fgets(buffer, MAX_BUFFER, fp);
-    if (feof(fp)) {
-      success = g_markup_parse_context_end_parse(parseContext,
-                                                 error);
-      break;
+    if (fgets(buffer, MAX_BUFFER, fp)) {
+      success = g_markup_parse_context_parse(parseContext,
+                                             buffer,
+                                             strlen(buffer),
+                                             error);
     }
 
-    success = g_markup_parse_context_parse(parseContext,
-                                           buffer,
-                                           strlen(buffer),
-                                           error);
+    if (feof(fp)) {
+      success &= g_markup_parse_context_end_parse(parseContext,
+                                                  error);
+      break;
+    }
     if (!success)
       break;
   }
@@ -603,6 +622,7 @@ xml_start_element(GMarkupParseContext *context,
         ++attribute_values;
       }
     } else if (strcmp(element_name, "port") == 0) {
+      parser->currText = &parser->world->port;
       parser->state = KC_PORT;
     } else if (strcmp(element_name, "charset") == 0) {
       parser->currText = &parser->world->charset;
@@ -630,6 +650,26 @@ xml_start_element(GMarkupParseContext *context,
         ++attribute_names;
         ++attribute_values;
       }
+    } else if (strcmp(element_name, "log") == 0) {
+      parser->state = KC_LOG;
+      if (parser->forWorldSelector) {
+        break;
+      }
+
+      while (*attribute_names) {
+        if (strcmp(*attribute_names, "autostart") == 0) {
+          parser->world->log_autostart = atoi(*attribute_values);
+        } else if (strcmp(*attribute_names, "addtime") == 0) {
+          parser->world->log_add_time = atoi(*attribute_values);
+        } else {
+          g_set_error(error,
+                      G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                      _("Invalid attribute '%s' for element '%s'"),
+                      *attribute_names, "log");
+        }
+        ++attribute_names;
+        ++attribute_values;
+      }
     } else if (strcmp(element_name, "auto-logon") == 0) {
       parser->state = KC_AUTO_LOGON;
       if (parser->forWorldSelector) {
@@ -707,6 +747,24 @@ xml_start_element(GMarkupParseContext *context,
         ++attribute_names;
         ++attribute_values;
       }
+    } else if (strcmp(element_name, "store-commands") == 0) {
+      parser->state = KC_STORE_COMMANDS;
+      if (parser->forWorldSelector) {
+        break;
+      }
+
+      while (*attribute_names) {
+        if (strcmp(*attribute_names, "enabled") == 0) {
+          parser->world->store_commands = atoi(*attribute_values);
+        } else {
+          g_set_error(error,
+                      G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                      _("Invalid attribute '%s' for element '%s'"),
+                      *attribute_names, "command-echo");
+        }
+        ++attribute_names;
+        ++attribute_values;
+      }
     } else if (strcmp(element_name, "command-history") == 0) {
       parser->state = KC_COMMAND_HISTORY;
       if (parser->forWorldSelector) {
@@ -963,10 +1021,29 @@ xml_start_element(GMarkupParseContext *context,
     }
     break;
 
+  case KC_LOG:
+    if (strcmp(element_name, "logfile") == 0) {
+      parser->currText = &parser->world->log_file_name;
+      parser->state = KC_LOGFILE;
+    } else if (strcmp(element_name, "logtimeformat") == 0) {
+      g_free(parser->world->log_timeformat);
+      parser->currText = &parser->world->log_timeformat;
+      parser->state = KC_LOGTIMEFORMAT;
+    } else {
+      g_set_error(error,
+                  G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                  _("Unknown element '%s'"), element_name);
+    }
+    break;
+
   case KC_PLUGINS:
     if (strcmp(element_name, "plugin") == 0) {
       parser->currText = &parser->currPluginFile;
       parser->state = KC_PLUGIN;
+    } else {
+      g_set_error(error,
+                  G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                  _("Unknown element '%s'"), element_name);
     }
     break;
 
@@ -1346,12 +1423,14 @@ xml_end_element(GMarkupParseContext *context,
   case KC_FLOOD_PREVENTION:
   case KC_REPEAT_COMMANDS:
   case KC_COMMAND_ECHO:
+  case KC_STORE_COMMANDS:
   case KC_COMMAND_HISTORY:
   case KC_AUTO_COMPLETION:
   case KC_COMMAND_SEPARATOR:
   case KC_INPUT_LINES:
   case KC_INPUT_SPELL:
   case KC_CONFIRM_DELETE:
+  case KC_LOG:
   case KC_PLUGIN_ITEMS:
   case KC_SCRIPTFILE:
   case KC_PLUGINS:
@@ -1386,6 +1465,11 @@ xml_end_element(GMarkupParseContext *context,
     parser->state = KC_DISPLAY;
     break;
 
+  case KC_LOGFILE:
+  case KC_LOGTIMEFORMAT:
+    parser->state = KC_LOG;
+    break;
+
   case KC_PLUGIN:
     if (!parser->forWorldSelector) {
       parser->world->startup_plugin_files
@@ -1519,11 +1603,14 @@ xml_text(GMarkupParseContext *context,
   switch (parser->state) {
   case KC_NAME:
   case KC_HOST:
+  case KC_PORT:
   case KC_CHARSET:
   case KC_TERMINALFONT:
   case KC_ENTRYFONT:
   case KC_STATUSFONT:
   case KC_FLOOD_PREVENTION:
+  case KC_LOGFILE:
+  case KC_LOGTIMEFORMAT:
   case KC_SCRIPTFILE:
   case KC_PLUGIN:
   case KC_PATTERN:
@@ -1536,12 +1623,6 @@ xml_text(GMarkupParseContext *context,
     copy_text(parser->currText, text, text_len);
     break;
 
-  case KC_PORT:
-    copy_text(&tmp, text, text_len);
-    parser->world->port = atoi(tmp);
-    g_free(tmp);
-    break;
-
   case KC_CHARACTER:
     /* When loading to get the names to compare against passed
        command-line options, characters need not be loaded. */
@@ -1640,6 +1721,7 @@ save_world_to_file(World *world)
                                             _("Could not open file '%s': %s"),
                                             world->file,
                                             g_strerror(errno));
+    gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Error"));
     gtk_dialog_run(GTK_DIALOG(dlg));
     gtk_widget_destroy(dlg);
     return;
@@ -1655,7 +1737,7 @@ save_world_to_file(World *world)
     fprintf_escaped(fp,
                     "  <name>%s</name>\n"
                     "  <host tls=\"%d\">%s</host>\n"
-                    "  <port>%d</port>\n",
+                    "  <port>%s</port>\n",
                     world->name,
                     world->use_tls,
                     world->host,
@@ -1717,12 +1799,14 @@ save_world_to_file(World *world)
   fprintf_escaped(fp,
                   "  <repeat-commands enabled=\"%d\" />\n"
                   "  <command-echo enabled=\"%d\" />\n"
+                  "  <store-commands enabled=\"%d\" />\n"
                   "  <command-history size=\"%d\" />\n"
                   "  <auto-completion enabled=\"%d\" minprefix=\"%d\" />\n"
                   "  <command-separator separator=\"%s\" />\n"
                   "  <input-lines size=\"%d\" save=\"%d\" />\n",
                   world->repeat_commands,
                   world->cmd_echo,
+                  world->store_commands,
                   world->commands_to_save,
                   world->autocompletion,
                   world->autocompl_minprefix,
@@ -1746,6 +1830,18 @@ save_world_to_file(World *world)
   fprintf(fp, "  <scroll on-output=\"%d\" lines=\"%ld\" />\n",
           world->scrollOutput, world->buffer_lines);
 
+  /* Logging */
+  fprintf(fp, "  <log autostart=\"%d\" addtime=\"%d\">\n",
+          world->log_autostart, world->log_add_time);
+  if (world->log_file_name) {
+    fprintf_escaped(fp, "    <logfile>%s</logfile>\n", world->log_file_name);
+  }
+  if (world->log_timeformat) {
+    fprintf_escaped(fp, "    <logtimeformat>%s</logtimeformat>\n",
+                    world->log_timeformat);
+  }
+  fprintf(fp, "  </log>\n");
+
   /* Delete confirmation in GUI Editors */
   fprintf(fp, "  <confirm-delete confirm=\"%d\" />\n",
           world->confirm_delete);
@@ -1897,6 +1993,9 @@ load_command_history(World *world)
   GtkListStore *liststore;
   GtkTreeIter   iter;
 
+  if (!world->store_commands)
+    return;
+
   if (!world->file) {
     return;
   }
@@ -1938,6 +2037,9 @@ save_command_history(World *world)
   gchar       *command;
   int          start_pos;
 
+  if (!world->store_commands)
+    return;
+
   if (!world->file) {
     fprintf(stderr, "Error: null filename\n");
     return;
