@@ -1,4 +1,4 @@
-/* $Id: worldselector.c,v 1.52 2007/01/28 11:46:17 ekalin Exp $ */
+/* $Id: worldselector.c,v 1.55 2007/11/25 18:37:05 ekalin Exp $ */
 
 /*
  * This program is free software; you can redistribute it and/or modify
@@ -119,7 +119,7 @@ get_world_to_connect()
                           wrld->name);
         wrld->host = g_strdup(gtk_entry_get_text(GTK_ENTRY(txtHost)));
         portstr = gtk_entry_get_text(GTK_ENTRY(txtPort));
-        wrld->port = atoi(portstr);
+        wrld->port = g_strdup(portstr);
         wrld->character_used = -1;    /* No auto-logon */
         success = TRUE;
       } else {
@@ -146,6 +146,7 @@ get_world_to_connect()
                                                        _("Could not load world from file '%s': %s"),
                                                        file,
                                                        error->message);
+            gtk_window_set_title(GTK_WINDOW(msgdlg), _("KildClient Error"));
             gtk_dialog_run(GTK_DIALOG(msgdlg));
             gtk_widget_destroy(msgdlg);
             free_world(wrld);
@@ -719,6 +720,7 @@ add_edit_buttons_cb(GtkButton *btn, gpointer data)
                                                  _("Could not load world from file '%s': %s"),
                                                  file,
                                                  error->message);
+      gtk_window_set_title(GTK_WINDOW(msgdlg), _("KildClient Error"));
       gtk_dialog_run(GTK_DIALOG(msgdlg));
       gtk_widget_destroy(msgdlg);
       g_free(world);
@@ -772,8 +774,7 @@ delete_button_cb(GtkButton *btn, gpointer data)
   char             *name;
   char             *file;
   GtkWidget        *msgdlg;
-  GtkWidget *dlgbutton;
-  GtkWidget *dlgbuttonimage;
+  gchar            *msg;
 
   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(lstWorlds));
   if (!gtk_tree_selection_get_selected(selection, &model, &iter))
@@ -789,28 +790,11 @@ delete_button_cb(GtkButton *btn, gpointer data)
                      COLUMN_FILE, &file,
                      -1);
 
-  msgdlg = gtk_message_dialog_new(GTK_WINDOW(wndMain),
-                                  GTK_DIALOG_DESTROY_WITH_PARENT,
-                                  GTK_MESSAGE_QUESTION,
-                                  GTK_BUTTONS_NONE,
-                                  _("Really delete world %s?"),
-                                  name);
-  gtk_window_set_title(GTK_WINDOW(msgdlg), _("Confirm deletion"));
-  dlgbutton = gtk_button_new_with_label(_("Do not delete"));
-  dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_NO,
-                                            GTK_ICON_SIZE_MENU);
-  gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-  gtk_widget_show_all(dlgbutton);
-  gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                               dlgbutton, GTK_RESPONSE_NO);
-
-  dlgbutton = gtk_button_new_with_label(_("Delete"));
-  dlgbuttonimage = gtk_image_new_from_stock(GTK_STOCK_DELETE,
-                                            GTK_ICON_SIZE_MENU);
-  gtk_button_set_image(GTK_BUTTON(dlgbutton), dlgbuttonimage);
-  gtk_widget_show_all(dlgbutton);
-  gtk_dialog_add_action_widget(GTK_DIALOG(msgdlg),
-                               dlgbutton, GTK_RESPONSE_YES);
+  msg = g_strdup_printf("world '%s'", name);
+  msgdlg = we_guied_confirm_delete_dialog_new(GTK_WINDOW(wndMain),
+                                              1,
+                                              msg, "");
+  g_free(msg);
 
   if (gtk_dialog_run(GTK_DIALOG(msgdlg)) == GTK_RESPONSE_YES) {
     GtkTreeIter  child;
@@ -855,6 +839,7 @@ disconnected_msgbox(gchar *msg, gboolean can_offline)
                                GTK_MESSAGE_QUESTION,
                                GTK_BUTTONS_NONE,
                                msg);
+  gtk_window_set_title(GTK_WINDOW(dlg), _("Disconnected"));
   if (can_offline) {
     gtk_dialog_add_buttons(GTK_DIALOG(dlg),
                            _("Reconnect"), RESP_RECONNECT,
